﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Web;

using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.Administration.Claims;
using Microsoft.SharePoint.Diagnostics;
using Microsoft.SharePoint.Utilities;

using Microsoft.IdentityModel;
using Microsoft.IdentityModel.Claims;
using Microsoft.SharePoint.WebControls;
using System.Data;
using System.Data.SqlClient;
using System.Collections;
using System.Configuration;
using System.Security.Permissions;

namespace Microsoft.Sample.ContentWeb.ClaimsSecurity
{
    [Microsoft.SharePoint.Security.SharePointPermission(System.Security.Permissions.SecurityAction.Demand, ObjectModel = true)]
    public class ContentWebClaimProvider : SPClaimProvider
    {
        private static string connectionString = null;


        public ContentWebClaimProvider(string displayName) : base(displayName)
        {
            Guid guid = new Guid(@"56705e15-abd3-44f0-adea-91488da1a572");
            ConnectionStringStorage storage = null;

            // Read Hierarchical Object Store set by PowerShell scripts.
            if (null != SPFarm.Local)
            {
                storage = (ConnectionStringStorage)SPFarm.Local.GetObject(guid);
                if (null != storage)
                {
                    connectionString = storage.ConnectionString;
                }
            }
        }

        protected override void FillClaimTypes(List<string> claimTypes)
        {
            if (null == claimTypes)
            {
                throw new ArgumentNullException("claimTypes");
            }

            // Add the claim types that will be added by this claims provider.  
            // NOTE: For every claim added here please make sure you add a corresponding claimValueTypes.Add line in FillClaimValueTypes method.
            claimTypes.Add(ContentWebClaimTypes.PersonnelNumber);
            claimTypes.Add(ContentWebClaimTypes.Alias);
            claimTypes.Add(ContentWebClaimTypes.FullName);
            claimTypes.Add(ContentWebClaimTypes.CompanyCode);
            claimTypes.Add(ContentWebClaimTypes.UserType);
            claimTypes.Add(ContentWebClaimTypes.UserSubType);
            claimTypes.Add(ContentWebClaimTypes.Manager);
            claimTypes.Add(ContentWebClaimTypes.HROrg);
            claimTypes.Add(ContentWebClaimTypes.HRPro);
            claimTypes.Add(ContentWebClaimTypes.Partner);
            claimTypes.Add(ContentWebClaimTypes.Leader);
            claimTypes.Add(ContentWebClaimTypes.VP);
            claimTypes.Add(ContentWebClaimTypes.CountryCode);
            claimTypes.Add(ContentWebClaimTypes.Level);
            claimTypes.Add(ContentWebClaimTypes.Profession);
            claimTypes.Add(ContentWebClaimTypes.Discipline);
            claimTypes.Add(ContentWebClaimTypes.StandardTitle);
            claimTypes.Add(ContentWebClaimTypes.Blocked);
            claimTypes.Add(ContentWebClaimTypes.BlockedRedirect);
            claimTypes.Add(ContentWebClaimTypes.VPAPersonnelNumber);
            claimTypes.Add(ContentWebClaimTypes.VPAAlias);
            claimTypes.Add(ContentWebClaimTypes.VPAFullName);
            claimTypes.Add(ContentWebClaimTypes.CompoundClaim);
        }

        protected override void FillClaimValueTypes(List<string> claimValueTypes)
        {
            if (null == claimValueTypes)
            {
                throw new ArgumentNullException("claimValueTypes");
            }

            // Add as many lines as ClaimTypes.Add lines listed in FillClaimTypes method...
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
            claimValueTypes.Add(Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
        }

        /// <summary>
        /// Gets the name from a fully qualified name.
        /// </summary>
        /// <param name="fullName">The full name. </param>
        /// <returns>a name string. </returns>
        private string GetName(string fullName)
        {
            string[] aName;
            char[] aDelims = new Char[] { '/', '\\' };

            aName = fullName.Split(aDelims);
            return aName[aName.GetUpperBound(0)];
        }


        protected override void FillClaimsForEntity(Uri context, SPClaim entity, List<SPClaim> claims)
        {
            if (null == entity)
            {
                throw new ArgumentNullException("entity");
            }
            if (null == claims)
            {
                throw new ArgumentNullException("claims");
            }

            SPClaim userIdClaim = SPClaimProviderManager.DecodeUserIdentifierClaim(entity);
            string userAlias = GetName(userIdClaim.Value.ToLowerInvariant());

            try
            {
                DataSet userClaimsDataSet = null;
                using (new SPMonitoredScope("ContentWeb.CCP.GetUserClaims", 5))
                {
                    // Retrieve user claims from database, two tables should be returned with the first one for simple claims, the second one for compound claims.
                    userClaimsDataSet = CustomClaimSourceDA.GetUserClaims(userAlias, connectionString);
                }

                // Add simple claims from Tables[0], ignore null values
                if (null != userClaimsDataSet && null != userClaimsDataSet.Tables[0] && userClaimsDataSet.Tables[0].Rows.Count > 0)
                {
                    foreach (DataRow claimsDataRow in userClaimsDataSet.Tables[0].Rows)
                    {
                        if (!DBNull.Value.Equals(claimsDataRow["PersonnelNumber"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.PersonnelNumber, (string)claimsDataRow["PersonnelNumber"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                            claims.Add(CreateClaim(ContentWebClaimTypes.Alias, userAlias, Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                        if (!DBNull.Value.Equals(claimsDataRow["FullName"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.FullName, (string)claimsDataRow["FullName"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                        if (!DBNull.Value.Equals(claimsDataRow["CompanyCode"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.CompanyCode, (string)claimsDataRow["CompanyCode"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                        if (!DBNull.Value.Equals(claimsDataRow["UserType"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.UserType, (string)claimsDataRow["UserType"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                        if (!DBNull.Value.Equals(claimsDataRow["UserSubType"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.UserSubType, (string)claimsDataRow["UserSubType"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                        if (!DBNull.Value.Equals(claimsDataRow["Manager"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.Manager, (string)claimsDataRow["Manager"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                        if (!DBNull.Value.Equals(claimsDataRow["HROrg"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.HROrg, (string)claimsDataRow["HROrg"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                        if (!DBNull.Value.Equals(claimsDataRow["HRPro"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.HRPro, (string)claimsDataRow["HRPro"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                        if (!DBNull.Value.Equals(claimsDataRow["Partner"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.Partner, (string)claimsDataRow["Partner"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                        if (!DBNull.Value.Equals(claimsDataRow["Leader"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.Leader, (string)claimsDataRow["Leader"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                        if (!DBNull.Value.Equals(claimsDataRow["VP"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.VP, (string)claimsDataRow["VP"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                        if (!DBNull.Value.Equals(claimsDataRow["CountryCode"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.CountryCode, (string)claimsDataRow["CountryCode"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                        if (!DBNull.Value.Equals(claimsDataRow["Level"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.Level, (string)claimsDataRow["Level"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                        if (!DBNull.Value.Equals(claimsDataRow["Profession"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.Profession, (string)claimsDataRow["Profession"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                        if (!DBNull.Value.Equals(claimsDataRow["Discipline"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.Discipline, (string)claimsDataRow["Discipline"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                        if (!DBNull.Value.Equals(claimsDataRow["StandardTitle"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.StandardTitle, (string)claimsDataRow["StandardTitle"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                        if (!DBNull.Value.Equals(claimsDataRow["Blocked"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.Blocked, (string)claimsDataRow["Blocked"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                        if (!DBNull.Value.Equals(claimsDataRow["BlockedRedirect"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.BlockedRedirect, (string)claimsDataRow["BlockedRedirect"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                        if (!DBNull.Value.Equals(claimsDataRow["VPAPersonnelNumber"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.VPAPersonnelNumber, (string)claimsDataRow["VPAPersonnelNumber"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                        if (!DBNull.Value.Equals(claimsDataRow["VPAAlias"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.VPAAlias, (string)claimsDataRow["VPAAlias"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                        if (!DBNull.Value.Equals(claimsDataRow["VPAFullName"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.VPAFullName, (string)claimsDataRow["VPAFullName"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                    }
                }

                // Add compound claims from Tables[1], ignore null values
                if (null != userClaimsDataSet && null != userClaimsDataSet.Tables[1] && userClaimsDataSet.Tables[1].Rows.Count > 0)
                {
                    foreach (DataRow claimsDataRow in userClaimsDataSet.Tables[1].Rows)
                    {
                        if (!DBNull.Value.Equals(claimsDataRow["CompoundClaim"]))
                        {
                            claims.Add(CreateClaim(ContentWebClaimTypes.CompoundClaim, (string)claimsDataRow["CompoundClaim"], Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                claims.Add(CreateClaim(ContentWebClaimTypes.Message, ex.Message + " " + ex.StackTrace, Microsoft.IdentityModel.Claims.ClaimValueTypes.String));
            }

        }

        protected override void FillEntityTypes(List<string> entityTypes)
        {
            entityTypes.Add(SPClaimEntityTypes.User);
        }

        // Used by People Picker in SP UI.
        protected override void FillHierarchy(Uri context, string[] entityTypes, string hierarchyNodeID, int numberOfLevels, Microsoft.SharePoint.WebControls.SPProviderHierarchyTree hierarchy)
        {
            if (!EntityTypesContain(entityTypes, SPClaimEntityTypes.User))
                return;

            switch (hierarchyNodeID)
            {
                case null:
                    // Only add those claims which need to be used for securing/targetting...
                    hierarchy.AddChild(new SPProviderHierarchyNode(this.Name, "CompanyCode", "CompanyCode", true));
                    hierarchy.AddChild(new SPProviderHierarchyNode(this.Name, "UserType", "UserType", true));
                    hierarchy.AddChild(new SPProviderHierarchyNode(this.Name, "UserSubType", "UserSubType", true));
                    hierarchy.AddChild(new SPProviderHierarchyNode(this.Name, "Manager", "Manager", true));
                    hierarchy.AddChild(new SPProviderHierarchyNode(this.Name, "HROrg", "HROrg", true));
                    hierarchy.AddChild(new SPProviderHierarchyNode(this.Name, "HRPro", "HRPro", true));
                    hierarchy.AddChild(new SPProviderHierarchyNode(this.Name, "Partner", "Partner", true));
                    hierarchy.AddChild(new SPProviderHierarchyNode(this.Name, "Leader", "Leader", true));
                    hierarchy.AddChild(new SPProviderHierarchyNode(this.Name, "MSAA", "MSAA", true));
                    hierarchy.AddChild(new SPProviderHierarchyNode(this.Name, "VP", "VP", true));
                    hierarchy.AddChild(new SPProviderHierarchyNode(this.Name, "CountryCode", "CountryCode", true));
                    hierarchy.AddChild(new SPProviderHierarchyNode(this.Name, "Level", "Level", true));
                    hierarchy.AddChild(new SPProviderHierarchyNode(this.Name, "CompoundClaim", "CompoundClaim", true));
                    break;

                default:
                    break;
            }
        }

        protected override void FillResolve(Uri context, string[] entityTypes, SPClaim resolveInput, List<Microsoft.SharePoint.WebControls.PickerEntity> resolved)
        {
            string claimTypeName = string.Empty;
            string claimValue = string.Empty;

            // Handles only ContentWeb claim types
            if (null != resolveInput && resolveInput.ClaimType.Contains(ContentWebClaimTypes.ContentWebClaimTypePrefix))
            {
                claimValue = resolveInput.Value.ToLower();
                claimTypeName = GetName(resolveInput.ClaimType);

                PickerEntity entity = CreatePickerEntity();
                entity.Claim = resolveInput;
                entity.Description = resolveInput.ClaimType + " = " + claimValue;
                entity.DisplayText = claimTypeName + " = " + claimValue;
                entity.EntityData[PeopleEditorEntityDataKeys.DisplayName] = claimTypeName + " = " + claimValue;
                entity.EntityType = SPClaimEntityTypes.User;
                entity.IsResolved = true;
                resolved.Add(entity);
            }
        }

        protected override void FillResolve(Uri context, string[] entityTypes, string resolveInput, List<Microsoft.SharePoint.WebControls.PickerEntity> resolved)
        {
        }

        protected override void FillSchema(Microsoft.SharePoint.WebControls.SPProviderSchema schema)
        {
            return;
        }

        protected override void FillSearch(Uri context, string[] entityTypes, string searchPattern, string hierarchyNodeID, int maxCount, Microsoft.SharePoint.WebControls.SPProviderHierarchyTree searchTree)
        {
            if (!EntityTypesContain(entityTypes, SPClaimEntityTypes.User))
                return;

            string claimValue = searchPattern.ToLower();

            string claimType = ContentWebClaimTypes.ContentWebClaimTypePrefix + @"/" + hierarchyNodeID;

            if (null != hierarchyNodeID && hierarchyNodeID.Length > 0)
            {
                PickerEntity entity = CreatePickerEntity();
                entity.Claim = CreateClaim(claimType, claimValue, Microsoft.IdentityModel.Claims.ClaimValueTypes.String);
                entity.Description = claimType + " = " + claimValue;
                entity.DisplayText = hierarchyNodeID + " = " + claimValue;
                entity.EntityData[PeopleEditorEntityDataKeys.DisplayName] = hierarchyNodeID + " = " + claimValue;
                entity.EntityType = SPClaimEntityTypes.User;
                entity.IsResolved = true;
                searchTree.AddEntity(entity);
            }
        }

        internal static string ProviderDisplayName
        {
            get { return @"ContentWeb Claim Provider"; }
        }

        internal static string ProviderInternalName
        {
            get { return @"ContentWebClaimProvider"; }
        }

        public override string Name
        {
            get { return ProviderInternalName; }
        }

        public override bool SupportsEntityInformation
        {
            get { return true; }
        }

        public override bool SupportsHierarchy
        {
            get { return true; }
        }

        public override bool SupportsResolve
        {
            get { return true; }
        }

        public override bool SupportsSearch
        {
            get { return true; }
        }
    }


}