﻿using System;
using System.Text;
using System.Web.UI;
using Microsoft.IdentityModel.Claims;
using Microsoft.SharePoint.Utilities;

namespace Microsoft.Sample.ContentWeb.ClaimsSecurity
{
    public partial class ClaimsView : UserControl
    {
        public override void RenderControl(HtmlTextWriter writer)
        {
            base.RenderControl(writer);
            try
            {
                using (SPMonitoredScope ClaimsScope =
                    new SPMonitoredScope("Microsoft.Sample.ContentWeb.WebParts.ClaimsWebPart.ClaimsWebPart"))
                {
                    //look for the claims identity
                    IClaimsPrincipal cp = Page.User as IClaimsPrincipal;

                    if (cp != null)
                    {
                        //get the claims identity so we can enum claims
                        IClaimsIdentity ci = (IClaimsIdentity)cp.Identity;

                        writer.AddStyleAttribute(HtmlTextWriterStyle.BorderStyle, "solid");
                        writer.AddStyleAttribute(HtmlTextWriterStyle.BorderWidth, "1px");
                        writer.RenderBeginTag(HtmlTextWriterTag.Table);
                        writer.RenderBeginTag(HtmlTextWriterTag.Tr);
                        writer.AddAttribute(HtmlTextWriterAttribute.Colspan, "2");
                        writer.AddStyleAttribute(HtmlTextWriterStyle.FontWeight, "bold");
                        writer.RenderBeginTag(HtmlTextWriterTag.Td);
                        writer.WriteEncodedText("The following claims were found in this request:");
                        writer.RenderEndTag(); // td
                        writer.RenderEndTag(); // tr

                        //enumerate all claims
                        foreach (Claim claim in ci.Claims)
                            RenderRow(writer, claim);

                        writer.RenderEndTag(); // table
                    }
                }
            }
            catch (Exception ex)
            {
                writer.Write("<b>There was an error rendering claims: " +
                    ex.Message + "</b>");
            }
        }

        private void RenderRow(HtmlTextWriter writer, Claim claim)
        {
            writer.RenderBeginTag(HtmlTextWriterTag.Tr);
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            writer.WriteEncodedText(StringTrim(claim.ClaimType.ToString(), 80));
            writer.RenderEndTag(); // td
            writer.RenderBeginTag(HtmlTextWriterTag.Td);
            writer.WriteEncodedText(StringTrim(claim.Value.ToString(), 60));
            writer.RenderEndTag(); // td
            writer.RenderEndTag(); // tr
        }

        private string StringTrim(string str, int length)
        {
            if (str.Length > length)
            {
                StringBuilder sbuilder = new StringBuilder(length * 2);
                sbuilder.Append(str.Substring(0, length - 3));
                sbuilder.Append("...");
                return sbuilder.ToString();
            }
            return str.PadRight(length, ' ');
        }
    }
}
