using System;
using System.Runtime.InteropServices;
using System.Security.Permissions;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Security;
using Microsoft.SharePoint.Administration.Claims;

namespace Microsoft.Sample.ContentWeb.ClaimsSecurity.Features.ClaimsSecurityFeature
{
    [SharePointPermission(System.Security.Permissions.SecurityAction.Demand, ObjectModel = true)]
    [Guid("760b783d-56b6-44ce-9e39-926a426121fb")]
    public class ClaimsSecurityFeatureEventReceiver : SPClaimProviderFeatureReceiver
    {
        public override string ClaimProviderAssembly
        {
            get { return typeof(ContentWebClaimProvider).Assembly.FullName; }
        }

        public override string ClaimProviderDescription
        {
            get { return @"This feature adds ContentWeb claim types/values in the security token"; }
        }

        public override string ClaimProviderDisplayName
        {
            get { return @"ContentWeb Claim Provider"; }
        }

        public override string ClaimProviderType
        {
            get { return typeof(ContentWebClaimProvider).FullName; }
        }

        /// <summary>
        /// This methods gets called after the activation of the feature.
        /// </summary>

        /// <param name="properties">Represents the properties of a feature activation<./param>
        /// <returns>void.</returns>
        public override void FeatureActivated(Microsoft.SharePoint.SPFeatureReceiverProperties properties)
        {
            ExecBaseFeatureActivated(properties);
        }

        /// <summary>
        /// Because use of base keywork can lead to unverifiable code inside lambda expression, 
        /// this function is created as a wrapper for the base.FeatureActivated function.
        /// This function gets called in the following lambda expression.
        /// </summary>

        /// <param name="properties">Represents the properties of a feature activation.</param>
        /// <returns> void </returns>
        private void ExecBaseFeatureActivated(Microsoft.SharePoint.SPFeatureReceiverProperties properties)
        {
            base.FeatureActivated(properties);
        }
    }
}
